@@ -6,48 +6,13 @@ Description
 import os, io, json, csv, zipfile
 import requests
 
-
-#  Note: need to import from package when running from wheel.
-# if running locally through main.py file, these imports should be replaced
-#       from . import settings
-# annoying, but it is what it is.
-from scrilla import settings
-
-import util.outputter as outputter
-import util.helper as helper
-
-CLOSE_PRICE = "close"
-OPEN_PRICE = "open"
-
-OBJECTS={
-    "correlation": 1,
-    "risk_profile": 2,
-    "prices": 3,
-    "dividends": 4,
-    "statistic": 5,
-    "equity_statistic":6,
-    "api_key": 7
-}
+from scrilla import settings, static, errors
+import scrilla.util.outputter as outputter
+import scrilla.util.helper as helper
 
 logger = outputter.Logger("files", settings.LOG_LEVEL)
 
-static_crypto_blob, static_econ_blob, static_tickers_blob = None, None, None
-
-object_blob_dict = {}
-
-def determine_analysis_date_range(start_date=None, end_date=None):
-    if end_date is None:
-        end_date = helper.get_previous_business_date(date=helper.get_today())
-    if start_date is None:
-        start_date = helper.decrement_date_by_business_days(end_date, settings.DEFAULT_ANALYSIS_PERIOD)
-    return start_date, end_date
-
-def generate_timestamp(args):
-    start_date = args.get('start_date')
-    end_date = args.get('end_date')
-    start_date, end_date = determine_analysis_date_range(start_date=start_date, end_date=end_date)
-    timestamp = '{}_{}'.format(helper.date_to_string(start_date),helper.date_to_string(end_date))
-    return timestamp
+static_tickers_blob, static_econ_blob, static_crypto_blob = None, None, None
 
 def load_file(file_name):
     with open(file_name, 'r') as infile:
@@ -63,185 +28,18 @@ def save_file(file_to_save, file_name):
                 json.dump(file_to_save, outfile)
                 return True
             except Exception as e:
-                logger.info(f'A {e.__class__} exception occured.')
+                logger.debug(f'A {e.__class__} exception occured.')
                 return False
             
         # TODO: implement other file saving extensions.
 
+def set_credentials(value, which_key):
+    file_name = os.path.join(settings.COMMON_DIR, f'{which_key}.{settings.FILE_EXT}')
+    return save_file(file_to_save=value, file_name=file_name)
 
-def store_local_object(local_object, value, args):
-    """
-    Parameters
-    ----------
-    1. local_object: int \n
-        index of object type you wish to store. Types are statically accessible through OBJECTS dictionary property. \n \n 
-
-    2. value: dict
-        dictionary of object key-value pairs to store. \n \n 
-
-    3. args: dict
-        dictionary of general and object-specific arguments. \n \n
-        
-        optional arguments: \n
-        1. end_date, start_date: datetime.date \n
-            range defaults to last 100 days if no dates are provided. \n \n 
-
-        required arguments: \n
-            NOTE: required arguments are a function of the object-type. \n 
-        1. if local_object = correlation \n 
-            args: { 'ticker_1' : value , 'ticker_2': value }  \n 
-        2. if local_object = prices \n 
-            args: { 'ticker' : value } \n 
-        3. if local_objecct = risk_profile \n 
-            args: { 'ticker' : value } \n
-        4. if local_object = dividends \n 
-            args: { 'ticker' : value } \n 
-        5. if local_object = statistic \n 
-            args: { 'stat_symbol' : value } \n 
-        6. if local_object = equity_statistic \n 
-            args: { 'ticker' : value , 'equity_stat_symbol' : value } \n \n
-        7. if local_object = api_key \n
-            args: { 'key_name': value, 'key_value': value }
-
-    """
-    global object_blob_dict
-
-    if settings.LOCAL_CACHE:
-        timestamp = generate_timestamp(args=args)
-        
-        file_name_2, in_memory_key = None, None
-
-        if local_object == OBJECTS['correlation']:
-            in_memory_key_1=f'{args["ticker_1"]}_{args["ticker_2"]}_{settings.CACHE_COR_KEY}'
-            in_memory_key_2=f'{args["ticker_2"]}_{args["ticker_1"]}_{settings.CACHE_COR_KEY}'
-            
-            logger.debug(f'Storing {in_memory_key_1} in-memory')
-            object_blob_dict[in_memory_key_1], object_blob_dict[in_memory_key_2] = value, value
-            
-            file_name = f'{timestamp}_{in_memory_key_1}.{settings.FILE_EXT}'
-            file_name_2 = f'{timestamp}_{in_memory_key_2}.{settings.FILE_EXT}'
-        
-        elif local_object == OBJECTS['prices']:
-            in_memory_key = f'{args["ticker"]}_{settings.CACHE_PRICE_KEY}'
-
-        elif local_object == OBJECTS['risk_profile']:
-            in_memory_key = f'{args["ticker"]}_{settings.CACHE_PRO_KEY}'
-
-        elif local_object == OBJECTS['dividends']:
-            in_memory_key=f'{args["ticker"]}_{settings.CACHE_DIV_KEY}'
-        
-        elif local_object == OBJECTS['statistic']:
-            in_memory_key = f'{args["stat_symbol"]}_{settings.CACHE_STAT_KEY}'
-        
-        elif local_object == OBJECTS['equity_statistic']:
-            in_memory_key = f'{args["ticker"]}_{args["equity_stat_symbol"]}_{settings.CACHE_EQUITY_KEY}'
-        
-        elif local_object == OBJECTS['api_key']:
-            file_name = os.path.join(settings.COMMON_DIR, f'{args["key_name"]}.{settings.FILE_EXT}')
-        
-
-        if in_memory_key is not None:
-            logger.debug(f'Storing {in_memory_key} in-memory')
-            object_blob_dict[in_memory_key] = value
-            file_name = os.path.join(settings.CACHE_DIR,f'{timestamp}_{in_memory_key}.{settings.FILE_EXT}')
-
-        if file_name_2 is not None:
-            buffer_store= os.path.join(settings.CACHE_DIR, file_name_2)
-            save_file(file_to_save=value, file_name=buffer_store)
-
-        if file_name is not None:
-            logger.debug(f'Storing {file_name} in local cache')
-            buffer_store= os.path.join(settings.CACHE_DIR, file_name)
-            return save_file(file_to_save=value, file_name=buffer_store) 
-
-        return None
-
-    return None
-
-def retrieve_local_object(local_object, args):
-    global object_blob_dict
-
-    if settings.LOCAL_CACHE:
-        timestamp = generate_timestamp(args=args)
-        in_memory_key = None
-        
-        if local_object == OBJECTS['correlation']:
-            in_memory_key_1=f'{args["ticker_1"]}_{args["ticker_2"]}_{settings.CACHE_COR_KEY}'
-            in_memory_key_2=f'{args["ticker_2"]}_{args["ticker_1"]}_{settings.CACHE_COR_KEY}'
-
-            logger.debug(f'Checking for {in_memory_key_1}')
-
-            if in_memory_key_1 in object_blob_dict.keys():
-                logger.debug(f'Loading in-memory {in_memory_key_1}')
-                return object_blob_dict[in_memory_key_1]
-            if in_memory_key_2 in object_blob_dict.keys():
-                logger.debug(f'Loading in-memomry {in_memory_key_2}')
-                return object_blob_dict[in_memory_key_2]
-
-            buffer_store_1= os.path.join(settings.CACHE_DIR, f'{timestamp}_{in_memory_key_1}.{settings.FILE_EXT}')
-            buffer_store_2= os.path.join(settings.CACHE_DIR, f'{timestamp}_{in_memory_key_2}.{settings.FILE_EXT}')
-
-            if os.path.isfile(buffer_store_1):
-                logger.debug(f'Loading in {in_memory_key_1} from local cache')
-                correlation = load_file(file_name=buffer_store_1)
-                object_blob_dict[in_memory_key_1] = correlation
-                return correlation
-
-            if os.path.isfile(buffer_store_2):
-                logger.debug(f'Loading in {in_memory_key_2} from local cache')
-                correlation = load_file(file_name=buffer_store_2)
-                object_blob_dict[in_memory_key_2] = correlation
-                return correlation
-            return None
-
-        elif local_object == OBJECTS['risk_profile']:
-            in_memory_key =f'{args["ticker"]}_{settings.CACHE_PRO_KEY}'
-        
-        elif local_object == OBJECTS['prices']:
-            # TODO: with different time stamp implementation, this part of the method should search for price histories that contain
-            # the range in start_date and end_date.
-
-            # start_date, end_date = args.get('start_date'), args.get('end_date')
-            # start_date, end_date = determine_analysis_date_range(start_date, end_date)
-
-            # for file in path(settings.CACHE_DIR) or whatever:
-                # key_check = f'{args["ticker"]}_{settings.CACHE_PRICE_KEY}'
-                # if key_check in file.name:
-                    # components = file.name.split('_')
-                    # cached_start_date = parsed_date(components[0])
-                    # cached_end_date = parsed_date(components[1])
-            
-                    # if cached_start_date <= start_date and cached_end_date >= end_date:
-                        # parsed start_date to end_date from cached_prices
-            in_memory_key=f'{args["ticker"]}_{settings.CACHE_PRICE_KEY}'
-
-        elif local_object == OBJECTS['dividends']:
-            in_memory_key=f'{args["ticker"]}_{settings.CACHE_DIV_KEY}'
-            
-        elif local_object == OBJECTS['statistic']:
-            in_memory_key=f'{args["stat_symbol"]}_{settings.CACHE_STAT_KEY}'
-        
-        elif local_object == OBJECTS['equity_statistic']:
-            in_memory_key=f'{args["ticker"]}_{args["equity_stat_symbol"]}_{settings.CACHE_EQUITY_KEY}'
-        
-        elif local_object == OBJECTS['api_key']:
-            file_name = os.path.join(settings.COMMON_DIR, f'{args["key_name"]}.{settings.FILE_EXT}')
-
-        if in_memory_key is not None:
-            logger.debug(f'Checking for {in_memory_key} data locally')
-
-            if in_memory_key in object_blob_dict.keys():
-                logger.debug(f'Found in-memory {in_memory_key}')
-                return object_blob_dict[in_memory_key]
-            file_name = os.path.join(settings.CACHE_DIR, f'{timestamp}_{in_memory_key}.{settings.FILE_EXT}')
-
-        if file_name is not None and os.path.isfile(file_name):
-            logger.debug(f'Loading {file_name} in local cache')
-            results = load_file(file_name = file_name)
-            object_blob_dict[in_memory_key] = results
-            return results
-
-    return None
+def get_credentials(which_key):
+    file_name = os.path.join(settings.COMMON_DIR, f'{which_key}.{settings.FILE_EXT}')
+    return load_file(file_name = file_name)
 
 def parse_csv_response_column(column, url, firstRowHeader=None, savefile=None, zipped=None):
     """
@@ -293,7 +91,12 @@ def init_static_data():
     """
     Description
     -----------
-    Initializes the three static files defined in  settings: `STATIC_TICKERS_FILE`, `STATIC_CRYPTO_FILE` and `STATIC_ECON_FILE`. The data for these files is retrieved from the service managers. While this function blurs the lines between file management and service management, the function has been included in the `files.py` module rather than the `services.py` module due the unique response types of static metadata. All metadata is returned a csv or zipped csvs. These responses require specialized functions. Moreover, these files should only be initialized the first time the application executes. Subsequent executions will refer to their cached versions residing in the local or containerized filesytems. 
+    Initializes the three static files defined in  settings: `STATIC_TICKERS_FILE`, `STATIC_CRYPTO_FILE` and `STATIC_ECON_FILE`. The data for these files is retrieved from the service managers. While this function blurs the lines between file management and service management, the function has been included in the `files.py` module rather than the `services.py` module due the unique response types of static metadata. All metadata is returned as a csv or zipped csvs. These responses require specialized functions. Moreover, these files should only be initialized the first time the application executes. Subsequent executions will refer to their cached versions residing in the local filesytems. 
+
+    Raises
+    ------
+    1. errors.ConfigurationError \n
+        If `scrilla.settings.PRICE_MANAGER` and `scrilla.settings.STAT_MANAGER` are not configured through the environment variables `PRICE_MANAGER` and `STAT_MANAGER`, the function will throw this error.
     """
     global static_tickers_blob
     global static_econ_blob
@@ -315,40 +118,35 @@ def init_static_data():
                 #       into services.py in the future. 
                 query=f'{settings.PARAM_AV_FUNC}={settings.ARG_AV_FUNC_EQUITY_LISTINGS}'
                 url = f'{settings.AV_URL}?{query}&{settings.PARAM_AV_KEY}={settings.AV_KEY}'
+                static_tickers_blob = parse_csv_response_column(column=0, url=url, savefile=settings.STATIC_TICKERS_FILE,
+                                                                    firstRowHeader=settings.AV_RES_EQUITY_KEY)
 
-                logger.debug(f'Preparsing to parse \'{settings.PRICE_MANAGER}\' Response to query: {query}')
-                static_tickers_blob = parse_csv_response_column(column=0, url=url, firstRowHeader=settings.AV_RES_EQUITY_KEY, 
-                                                        savefile=settings.STATIC_TICKERS_FILE)
-
-            else:
-                logger.info("No PRICE_MANAGER set in .env file!")
+            raise errors.ConfigurationError("No PRICE_MANAGER set in .env file!")
 
         # grab crypto symbols and store in STATIC_DIR
         if not os.path.isfile(settings.STATIC_CRYPTO_FILE):
             if settings.PRICE_MANAGER == "alpha_vantage": 
                 logger.debug(f'Missing {settings.STATIC_CRYPTO_FILE}, querying \'{settings.PRICE_MANAGER}\'.')
                 url = settings.AV_CRYPTO_LIST
-
-                logger.debug(f'Preparsing to parse \'{settings.PRICE_MANAGER}\' Response to query: {query}')
-                static_crypto_blob = parse_csv_response_column(column=0, url=url, firstRowHeader=settings.AV_RES_CRYPTO_KEY, 
-                                                    savefile=settings.STATIC_CRYPTO_FILE)
-            else:
-                logger.info("No PRICE_MANAGER set in .env file!")
+                static_crypto_blob = parse_csv_response_column(column=0, url=url, savefile=settings.STATIC_CRYPTO_FILE,
+                                                                firstRowHeader=settings.AV_RES_CRYPTO_KEY)
+            raise errors.ConfigurationError("No PRICE_MANAGER set in .env file!")
             
         # grab econominc indicator symbols and store in STATIC_DIR
         if not os.path.isfile(settings.STATIC_ECON_FILE):
             if settings.STAT_MANAGER == "quandl":
+                service_map = static.keys["SERVICES"]["STATISTICS"]["QUANDL"]["MAP"]
+
                 logger.debug(f'Missing {settings.STATIC_ECON_FILE}, querying \'{settings.STAT_MANAGER}\'.')
 
-                query = f'{settings.PATH_Q_FRED}/{settings.PARAM_Q_METADATA}'
-                url = f'{settings.Q_META_URL}/{query}?{settings.PARAM_Q_KEY}={settings.Q_KEY}'
+                query = f'{service_map["PATHS"]["FRED"]}/{service_map["PARAMS"]["METADATA"]}'
+                url = f'{settings.Q_META_URL}/{query}?{service_map["PARAMS"]["KEY"]}={settings.Q_KEY}'
+                static_econ_blob = parse_csv_response_column(column=0, url=url, savefile=settings.STATIC_ECON_FILE,
+                                                            firstRowHeader=service_map["KEYS"]["HEADER"],
+                                                             zipped=service_map["KEYS"]["ZIPFILE"])
 
-                logger.debug(f'Preparsing to parse \'{settings.PRICE_MANAGER}\' Response to query: {query}')
-                static_econ_blob = parse_csv_response_column(column=0, url=url, firstRowHeader=settings.Q_RES_STAT_KEY,
-                                                    savefile=settings.STATIC_ECON_FILE, zipped=settings.Q_RES_STAT_ZIP_KEY)
+            raise errors.ConfigurationError("No STAT_MANAGER set in .env file!")
 
-            else:
-                logger.info("No STAT_MANAGER set in .env file!")
     else:
         logger.debug('Static data already initialized!')
 
@@ -368,19 +166,19 @@ def get_static_data(static_type):
     global static_econ_blob 
     global static_tickers_blob
 
-    if static_type == settings.ASSET_CRYPTO:
+    if static_type == static.keys['ASSETS']['CRYPTO']:
         if static_crypto_blob is not None:
             blob = static_crypto_blob
         else:
             path = settings.STATIC_CRYPTO_FILE
     
-    elif static_type == settings.ASSET_EQUITY:
+    elif static_type == static.keys['ASSETS']['EQUITY']:
         if static_tickers_blob:
             blob = static_tickers_blob
         else:
             path = settings.STATIC_TICKERS_FILE
     
-    elif static_type == settings.STAT_ECON:
+    elif static_type == static.keys['ASSETS']['STAT']:
         if static_econ_blob:
             blob = static_econ_blob
         else:
@@ -403,11 +201,11 @@ def get_static_data(static_type):
                 symbols = json.load(infile)
             # TODO: implement other file loading exts 
                
-        if static_type == settings.ASSET_CRYPTO:
+        if static_type == static.keys['ASSETS']['CRYPTO']:
             static_crypto_blob = symbols
-        elif static_type == settings.ASSET_EQUITY:
+        elif static_type == static.keys['ASSETS']['EQUITY']:
             static_tickers_blob = symbols
-        elif static_type == settings.STAT_ECON:
+        elif static_type == static.keys['ASSETS']['STAT']:
             static_econ_blob = symbols
         return symbols
         
@@ -431,9 +229,9 @@ def get_overlapping_symbols(equities=None, cryptos=None):
     Returns an array of symbols which are contained in both the STATIC_TICKERS_FILE and STATIC_CRYPTO_FILE, i.e. ticker symbols which have both a tradeable equtiy and a tradeable crypto asset. 
     """
     if equities is None:
-        equities = list(get_static_data(settings.ASSET_EQUITY))
+        equities = list(get_static_data(static.keys['ASSETS']['EQUITY']))
     if cryptos is None:
-        cryptos = list(get_static_data(settings.ASSET_CRYPTO))
+        cryptos = list(get_static_data(static.keys['ASSETS']['CRYPTO']))
     overlap = []
     for crypto in cryptos:
         if crypto in equities:
@@ -450,24 +248,24 @@ def get_asset_type(symbol):
     ------
     A string representing the type of asset of the symbol. Types are statically accessible through the ` settings` variables: ASSET_EQUITY and ASSET_CRYPTO. \n \n 
     """
-    symbols = list(get_static_data(settings.ASSET_CRYPTO))
+    symbols = list(get_static_data(static.keys['ASSETS']['CRYPTO']))
     overlap = get_overlapping_symbols(cryptos=symbols)
 
     if symbol not in overlap:
         if symbol in symbols:
-            return settings.ASSET_CRYPTO
+            return static.keys['ASSETS']['CRYPTO']
             
                 # if other asset types are introduced, then uncomment these lines
                 # and add new asset type to conditional. Keep in mind the static
                 # equity data is HUGE.
-        # symbols = list(get_static_data(settings.ASSET_EQUITY))
+        # symbols = list(get_static_data(static.keys['ASSETS']['EQUITY']))
         # if symbol in symbols:
-            # return settings.ASSET_EQUITY
+            # return static.keys['ASSETS']['EQUITY']
         #return None
-        return settings.ASSET_EQUITY
+        return static.keys['ASSETS']['EQUITY']
     # default to equity for overlap until a better method is determined. 
-    return settings.ASSET_EQUITY
-
+    return static.keys['ASSETS']['EQUITY']
+    
 def get_watchlist():
     """
     Description
@@ -499,7 +297,7 @@ def add_watchlist(new_tickers):
     logger.debug('Saving tickers to Watchlist')
 
     current_tickers = get_watchlist()
-    all_tickers = get_static_data(settings.ASSET_EQUITY)
+    all_tickers = get_static_data(static.keys['ASSETS']['EQUITY'])
 
     for ticker in new_tickers:
         if ticker not in current_tickers and ticker in all_tickers:
@@ -534,11 +332,11 @@ def format_allocation(allocation, portfolio, investment=None):
     for j, item in enumerate(portfolio.tickers):
         holding = {}
         holding['ticker'] = item
-        holding['allocation'] = round(allocation[j], settings.ACCURACY)
+        holding['allocation'] = round(allocation[j], static.constants['ACCURACY'])
         if investment is not None:
             holding['shares'] = float(shares[j])
-        holding['annual_return'] = round(portfolio.mean_return[j], settings.ACCURACY) 
-        holding['annual_volatility'] = round(portfolio.sample_vol[j], settings.ACCURACY)
+        holding['annual_return'] = round(portfolio.mean_return[j], static.constants['ACCURACY']) 
+        holding['annual_volatility'] = round(portfolio.sample_vol[j], static.constants['ACCURACY'])
         allocation_format.append(holding)
 
     json_format = {}
@@ -643,7 +441,7 @@ def clear_directory(directory, retain=True):
 
     for f in filelist:
         filename = os.path.basename(f)
-        if retain and filename == settings.KEEP_EXT:
+        if retain and filename == static.constants['KEEP_FILE']:
             continue
         os.remove(os.path.join(directory, f))
 
