@@ -1,4 +1,5 @@
 import datetime
+from scrilla import static
 import util.helper as helper
 import util.outputter as outputter
 
@@ -167,8 +168,8 @@ class Cashflow:
         model_prices= self.generate_model_series()
 
         return[ { 'date': date, 
-                  'model_price': self.sample[date], 
-                  'actual_price': model_prices[index] } 
+                  'model_price': model_prices[index], 
+                  'actual_price': self.sample[date] } 
                 for index, date in enumerate(self.sample.keys()) ]
         
     def get_growth_function(self, x):
@@ -216,7 +217,7 @@ class Cashflow:
             
             self.NPV += self.get_growth_function(current_time) / ((1 + self.discount_rate)**current_time)
 
-            if self.NPV - previous_value < settings.NPV_DELTA_TOLERANCE:
+            if self.NPV - previous_value < static.constants['NPV_DELTA_TOLERANCE']:
                 calculating = False
             i += 1
 
